'use strict';

exports.__esModule = true;
exports.scrollbarSize = exports.keyCodes = undefined;
exports.getMonth = getMonth;
exports.getWeeksInMonth = getWeeksInMonth;
exports.getScrollSpeed = getScrollSpeed;
exports.emptyFn = emptyFn;

var _scrollbarSize = require('dom-helpers/util/scrollbarSize');

var _scrollbarSize2 = _interopRequireDefault(_scrollbarSize);

var _get_days_in_month = require('date-fns/get_days_in_month');

var _get_days_in_month2 = _interopRequireDefault(_get_days_in_month);

var _get_day = require('date-fns/get_day');

var _get_day2 = _interopRequireDefault(_get_day);

var _invariant = require('invariant');

var _invariant2 = _interopRequireDefault(_invariant);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var keyCodes = exports.keyCodes = {
  enter: 13,
  left: 37,
  up: 38,
  right: 39,
  down: 40,
  shift: 16,
  control: 17,
  command: 91,
  escape: 27
};

function getMonth(year, month, weekStartsOn) {
  var rows = [];
  var monthDate = new Date(year, month, 1);
  var daysInMonth = (0, _get_days_in_month2.default)(monthDate);
  var weekEndsOn = getEndOfWeekIndex(weekStartsOn);

  var dow = (0, _get_day2.default)(new Date(year, month, 1));
  var week = 0;

  for (var day = 1; day <= daysInMonth; day++) {
    if (!rows[week]) {
      rows[week] = [];
    }

    rows[week].push(day);

    if (dow === weekEndsOn) {
      week++;
    }

    dow = dow < 6 ? dow + 1 : 0;
  }

  return {
    date: monthDate,
    rows: rows
  };
}

function getWeek(year, date, weekStartsOn) {
  var yearStart = new Date(year, 0, 1); // 1st Jan of the Year

  return Math.ceil(((date - yearStart) / 86400000 + yearStart.getDay() + 1 - weekStartsOn) / 7);
}

function getWeeksInMonth(month) {
  var year = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : new Date().getFullYear();
  var weekStartsOn = arguments[2];

  var weekEndsOn = getEndOfWeekIndex(weekStartsOn);

  var firstOfMonth = new Date(year, month, 1);
  var firstWeekNumber = getWeek(year, firstOfMonth, weekStartsOn);

  var lastOfMonth = new Date(year, month + 1, 0); // Last date of the Month
  var lastWeekNumber = getWeek(year, lastOfMonth, weekStartsOn);

  var rowCount = lastWeekNumber - firstWeekNumber;

  // If the last week contains 7 days, we need to add an extra row
  if (lastOfMonth.getDay() === weekEndsOn) {
    rowCount++;
  }

  return rowCount;
}

var daysOfWeek = [0, 1, 2, 3, 4, 5, 6];

function getEndOfWeekIndex(weekStartsOn) {
  (0, _invariant2.default)(daysOfWeek.includes(weekStartsOn), 'Day of the week should be between 0 and 6');

  var weekEndsOn = weekStartsOn === 0 ? 6 : weekStartsOn - 1;

  return weekEndsOn;
}

function getScrollSpeed() {
  var settings = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {
    delay: 50 // in ms, higher means lower fidelity
  };

  var lastPos = void 0;
  var newPos = void 0;
  var timer = void 0;
  var delta = void 0;

  function clear() {
    lastPos = null;
    delta = 0;
  }

  clear();

  return function (scrollY) {
    newPos = scrollY;
    if (lastPos != null) {
      delta = newPos - lastPos;
    }
    lastPos = newPos;
    clearTimeout(timer);
    timer = setTimeout(clear, settings.delay);
    return delta;
  };
}

var scrollbarSize = exports.scrollbarSize = (0, _scrollbarSize2.default)();

function emptyFn() {/* no-op */}